from django.conf.urls import patterns, url

from .views import IndexView
from .views import SurveyDetail
from .views import ConfirmView
from .views import SurveyCompleted

urlpatterns = patterns(
    '',
    url(r'^$', IndexView.as_view(), name='survey-list'),
    url(
        r'^survey/(?P<id>\d+)/$',
        SurveyDetail.as_view(),
        name='survey-detail'
    ),
    url(
        r'^survey/(?P<id>\d+)/completed/$',
        SurveyCompleted.as_view(),
        name='survey-completed'
    ),
    url(
        r'^survey/(?P<id>\d+)-(?P<step>\d+)/$',
        SurveyDetail.as_view(),
        name='survey-detail-step'
    ),
    url(
        r'^confirm/(?P<uuid>\w+)/$',
        ConfirmView.as_view(),
        name='survey-confirmation'
    ),
)



from django.utils.translation import ungettext, ugettext_lazy


# Actions
def make_published(modeladmin, request, queryset):
    """
    Mark the given survey as published
    """
    count = queryset.update(is_published=True)
    message = ungettext(
        u'%(count)d survey was successfully marked as published.',
        u'%(count)d surveys were successfully marked as published',
        count
    ) % {'count': count}
    modeladmin.message_user(request, message)
make_published.short_description = ugettext_lazy(
    u"Mark selected surveys as published"
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='AnswerBase',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=400)),
                ('order', models.IntegerField(null=True, blank=True)),
            ],
            options={
                'verbose_name': 'category',
                'verbose_name_plural': 'categories',
            },
        ),
        migrations.CreateModel(
            name='Question',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('text', models.TextField()),
                ('order', models.IntegerField()),
                ('required', models.BooleanField()),
                ('question_type', models.CharField(default=b'text', max_length=200, choices=[(b'text', 'text (multiple line)'), (b'short-text', 'short text (one line)'), (b'radio', 'radio'), (b'select', 'select'), (b'select-multiple', 'Select Multiple'), (b'select_image', 'Select Image'), (b'integer', 'integer')])),
                ('choices', models.TextField(help_text="if the question type is 'radio', 'select', or 'select multiple' provide a comma-separated list of options for this question .", null=True, blank=True)),
                ('category', models.ForeignKey(blank=True, to='survey.Category', null=True)),
            ],
            options={
                'ordering': ('survey', 'order'),
                'verbose_name': 'question',
                'verbose_name_plural': 'questions',
            },
        ),
        migrations.CreateModel(
            name='Response',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('interview_uuid', models.CharField(max_length=36, verbose_name='Interview unique identifier')),
            ],
            options={
                'verbose_name': 'response',
                'verbose_name_plural': 'responses',
            },
        ),
        migrations.CreateModel(
            name='Survey',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=400)),
                ('description', models.TextField()),
                ('is_published', models.BooleanField()),
                ('need_logged_user', models.BooleanField()),
                ('display_by_question', models.BooleanField()),
            ],
            options={
                'verbose_name': 'survey',
                'verbose_name_plural': 'surveys',
            },
        ),
        migrations.CreateModel(
            name='AnswerInteger',
            fields=[
                ('answerbase_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='survey.AnswerBase')),
                ('body', models.IntegerField(null=True, blank=True)),
            ],
            bases=('survey.answerbase',),
        ),
        migrations.CreateModel(
            name='AnswerRadio',
            fields=[
                ('answerbase_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='survey.AnswerBase')),
                ('body', models.TextField(null=True, blank=True)),
            ],
            bases=('survey.answerbase',),
        ),
        migrations.CreateModel(
            name='AnswerSelect',
            fields=[
                ('answerbase_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='survey.AnswerBase')),
                ('body', models.TextField(null=True, blank=True)),
            ],
            bases=('survey.answerbase',),
        ),
        migrations.CreateModel(
            name='AnswerSelectMultiple',
            fields=[
                ('answerbase_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='survey.AnswerBase')),
                ('body', models.TextField(null=True, blank=True)),
            ],
            bases=('survey.answerbase',),
        ),
        migrations.CreateModel(
            name='AnswerText',
            fields=[
                ('answerbase_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='survey.AnswerBase')),
                ('body', models.TextField(null=True, blank=True)),
            ],
            bases=('survey.answerbase',),
        ),
        migrations.AddField(
            model_name='response',
            name='survey',
            field=models.ForeignKey(to='survey.Survey'),
        ),
        migrations.AddField(
            model_name='response',
            name='user',
            field=models.ForeignKey(blank=True, to=settings.AUTH_USER_MODEL, null=True),
        ),
        migrations.AddField(
            model_name='question',
            name='survey',
            field=models.ForeignKey(to='survey.Survey'),
        ),
        migrations.AddField(
            model_name='category',
            name='survey',
            field=models.ForeignKey(to='survey.Survey'),
        ),
        migrations.AddField(
            model_name='answerbase',
            name='question',
            field=models.ForeignKey(to='survey.Question'),
        ),
        migrations.AddField(
            model_name='answerbase',
            name='response',
            field=models.ForeignKey(to='survey.Response'),
        ),
    ]



from django.contrib import admin

from .models import Question, Category, Survey, Response, AnswerText
from .models import AnswerRadio, AnswerSelect
from .models import AnswerInteger, AnswerSelectMultiple
from .actions import make_published


class QuestionInline(admin.TabularInline):
    model = Question
    ordering = ('category', 'order',)
    extra = 1


class CategoryInline(admin.TabularInline):
    model = Category
    extra = 0


class SurveyAdmin(admin.ModelAdmin):
    list_display = ('name', 'is_published', 'need_logged_user', 'template')
    list_filter = ('is_published', 'need_logged_user')
    inlines = [CategoryInline, QuestionInline]
    actions = [make_published]


class AnswerBaseInline(admin.StackedInline):
    fields = ('question', 'body')
    readonly_fields = ('question',)
    extra = 0


class AnswerTextInline(AnswerBaseInline):
    model = AnswerText


class AnswerRadioInline(AnswerBaseInline):
    model = AnswerRadio


class AnswerSelectInline(AnswerBaseInline):
    model = AnswerSelect


class AnswerSelectMultipleInline(AnswerBaseInline):
    model = AnswerSelectMultiple


class AnswerIntegerInline(AnswerBaseInline):
    model = AnswerInteger


class ResponseAdmin(admin.ModelAdmin):
    list_display = ('interview_uuid', 'survey', 'created', 'user')
    list_filter = ('survey', 'created')
    date_hierarchy = 'created'
    inlines = [
        AnswerTextInline, AnswerRadioInline, AnswerSelectInline,
        AnswerSelectMultipleInline, AnswerIntegerInline
    ]
    # specifies the order as well as which fields to act on
    readonly_fields = (
        'survey', 'created', 'updated', 'interview_uuid', 'user'
    )


admin.site.register(Survey, SurveyAdmin)
admin.site.register(Response, ResponseAdmin)



import uuid
import logging

from django import forms
from django.forms import models
from django.core.urlresolvers import reverse
from django.utils.safestring import mark_safe
from django.conf import settings

from survey.models import AnswerInteger, AnswerSelectMultiple
from survey.models import AnswerText, AnswerRadio, AnswerSelect
from survey.models import Question, Response
from survey.signals import survey_completed
from survey.utils import get_choices
from survey.widgets import ImageSelectWidget


class HorizontalRadioRenderer(forms.RadioSelect.renderer):
    def render(self):
        return mark_safe(u'\n'.join([u'%s\n' % w for w in self]))


class ResponseForm(models.ModelForm):
    class Meta:
        model = Response
        fields = ()

    def __init__(self, *args, **kwargs):
        # expects a survey object to be passed in initially
        separator = getattr(settings, 'SURVEY_SEPARATOR', ',')
        empty_tuple = ('', '-------------')
        survey = kwargs.pop('survey')
        self.survey = survey
        self.user = kwargs.pop('user')
        try:
            self.step = int(kwargs.pop('step'))
        except KeyError:
            self.step = None

        super(ResponseForm, self).__init__(*args, **kwargs)
        random_uuid = uuid.uuid4().hex
        self.uuid = random_uuid

        self.steps_count = survey.questions().count()
        # add a field for each survey question, corresponding to the question
        # type as appropriate.
        data = kwargs.get('data')
        for index, q in enumerate(survey.questions()):
            if (self.survey.display_by_question and
               index != self.step and self.step is not None):
                continue
            else:
                field_name = "question_%d" % q.pk
                if q.question_type == Question.TEXT:
                    self.fields[field_name] = forms.CharField(
                        label=q.text,
                        widget=forms.Textarea
                    )
                elif q.question_type == Question.SHORT_TEXT:
                    self.fields[field_name] = forms.CharField(
                        label=q.text,
                        widget=forms.TextInput
                    )
                elif q.question_type == Question.RADIO:
                    question_choices = get_choices(q.choices,
                                                   separator=separator)
                    self.fields[field_name] = forms.ChoiceField(
                        label=q.text,
                        widget=forms.RadioSelect(
                            renderer=HorizontalRadioRenderer
                        ),
                        choices=question_choices
                    )
                elif q.question_type == Question.SELECT:
                    question_choices = get_choices(q.choices,
                                                   separator=separator)
                    # add an empty option at the top so that the user has to
                    # explicitly select one of the options
                    question_choices = tuple([empty_tuple]) + question_choices
                    self.fields[field_name] = forms.ChoiceField(
                        label=q.text,
                        widget=forms.Select,
                        choices=question_choices
                    )
                elif q.question_type == Question.SELECT_IMAGE:
                    question_choices = get_choices(q.choices,
                                                   separator=separator)
                    # add an empty option at the top so that the user has to
                    # explicitly select one of the options
                    question_choices = tuple([empty_tuple]) + question_choices
                    self.fields[field_name] = forms.ChoiceField(
                        label=q.text,
                        widget=ImageSelectWidget,
                        choices=question_choices
                    )
                elif q.question_type == Question.SELECT_MULTIPLE:
                    question_choices = get_choices(q.choices,
                                                   separator=separator)
                    self.fields[field_name] = forms.MultipleChoiceField(
                        label=q.text,
                        widget=forms.CheckboxSelectMultiple,
                        choices=question_choices
                    )
                elif q.question_type == Question.INTEGER:
                    self.fields[field_name] = forms.IntegerField(label=q.text)
                # if the field is required, give it a corresponding css class.
                if q.required:
                    self.fields[field_name].required = True
                    self.fields[field_name].widget.attrs["class"] = "required"
                    self.fields[field_name].widget.attrs["required"] = True
                else:
                    self.fields[field_name].required = False
                # add the category as a css class, and add it as a data
                # attribute as well (this is used in the template to allow
                # sorting the questions by category)
                if q.category:
                    cat_name = q.category.name
                    classes = self.fields[field_name].widget.attrs.get("class")
                    category_class = " cat_%s" % q.category.name
                    if classes:
                        new_classes = classes + (category_class)
                    else:
                        new_classes = (category_class)
                    self.fields[field_name].widget.attrs["class"] = new_classes
                    self.fields[field_name].widget.attrs["category"] = cat_name

                classes = self.fields[field_name].widget.attrs.get("class")
                if q.question_type == Question.SELECT:
                    new_classes = classes + (" cs-select cs-skin-boxes")
                elif q.question_type == Question.RADIO:
                    new_classes = classes + (
                        " fs-radio-group fs-radio-custom clearfix")
                # elif q.question_type == Question.SELECT_MULTIPLE:
                #    new_classes = classes + (" ")
                self.fields[field_name].widget.attrs["class"] = new_classes

                # initialize the form field with values from a POST request, if
                # any.
                if data:
                    self.fields[field_name].initial = data.get(field_name)

    def has_next_step(self):
        """Check if the form has a next step."""
        if self.survey.display_by_question:
            if self.step < self.steps_count-1:
                return True
        return False

    def next_step_url(self):
        """Return the url for the next step."""
        if self.has_next_step():
            return reverse('survey-detail-step',
                           kwargs={
                               'id': self.survey.id,
                               'step': self.step+1
                           })
        else:
            return None

    def current_step_url(self):
        """Return the url for the current step."""
        return reverse('survey-detail-step',
                       kwargs={
                           'id': self.survey.id,
                           'step': self.step
                       })

    def save(self, commit=True):
        # save the response object
        response = super(ResponseForm, self).save(commit=False)
        response.survey = self.survey
        response.interview_uuid = self.uuid
        if self.user.is_authenticated():
            response.user = self.user
        response.save()

        # response "raw" data as dict (for signal)
        data = {
            'survey_id': response.survey.id,
            'interview_uuid': response.interview_uuid,
            'responses': []
        }
        # create an answer object for each question and associate it with this
        # response.
        for field_name, field_value in self.cleaned_data.iteritems():
            if field_name.startswith("question_"):
                # warning: this way of extracting the id is very fragile and
                # entirely dependent on the way the question_id is encoded in
                # the field name in the __init__ method of this form class.
                q_id = int(field_name.split("_")[1])
                q = Question.objects.get(pk=q_id)

                if q.question_type in [Question.TEXT, Question.SHORT_TEXT]:
                    a = AnswerText(question=q)
                    a.body = field_value
                elif q.question_type == Question.RADIO:
                    a = AnswerRadio(question=q)
                    a.body = field_value
                elif q.question_type == Question.SELECT:
                    a = AnswerSelect(question=q)
                    a.body = field_value
                elif q.question_type == Question.SELECT_IMAGE:
                    a = AnswerSelect(question=q)
                    value, img_src = field_value.split(":", 1)
                    a.body = value
                elif q.question_type == Question.SELECT_MULTIPLE:
                    a = AnswerSelectMultiple(question=q)
                    a.body = field_value
                elif q.question_type == Question.INTEGER:
                    a = AnswerInteger(question=q)
                    a.body = field_value
                data['responses'].append((a.question.id, a.body))
                # Some debug info
                msg = "creating answer to question %d of type %s" % (
                    q_id, a.question.question_type
                )
                logging.debug(msg)
                logging.debug(a.question.text)
                logging.debug('answer value:')
                logging.debug(field_value)

                a.response = response
                a.save()
        survey_completed.send(sender=Response, instance=response, data=data)
        return response



from django.shortcuts import render
from django.shortcuts import get_object_or_404
from django.shortcuts import redirect
from django.conf import settings

from .models import Survey, Category, Response
from .forms import ResponseForm


from django.views.generic import TemplateView, View


class IndexView(TemplateView):
    template_name = "survey/list.html"

    def get_context_data(self, **kwargs):
        context = super(IndexView, self).get_context_data(**kwargs)
        qs = Survey.objects.filter(is_published=True)
        if not self.request.user.is_authenticated():
            qs = qs.filter(need_logged_user=False)
        context['surveys'] = qs
        return context


class SurveyDetail(View):

    def get_survey_categories(self, survey):
        return Category.objects.filter(survey=survey).order_by('order')

    def get_user_response(self, user, survey):
        return Response.objects.filter(survey=survey, user=user)

    def get(self, request, *args, **kwargs):
        survey = get_object_or_404(Survey, is_published=True, id=kwargs['id'])
        if survey.template is not None and len(survey.template) > 4:
            template_name = survey.template
        else:
            if survey.display_by_question:
                template_name = 'survey/survey.html'
            else:
                template_name = 'survey/one_page_survey.html'
        if survey.need_logged_user and not request.user.is_authenticated():
            return redirect('%s?next=%s' % (settings.LOGIN_URL, request.path))
        if survey.need_logged_user and request.user.is_authenticated():
            if self.get_user_response(request.user, survey).exists():
                return redirect('survey-completed', id=survey.id)

        category_items = self.get_survey_categories(survey)
        categories = [c.name for c in category_items]
        step = kwargs.get('step', 0)
        form = ResponseForm(survey=survey, user=request.user, step=step)
        context = {
            'response_form': form,
            'survey': survey,
            'categories': categories,
        }

        return render(request, template_name, context)

    def post(self, request, *args, **kwargs):
        survey = get_object_or_404(Survey, is_published=True, id=kwargs['id'])
        if survey.need_logged_user and not request.user.is_authenticated():
            return redirect('%s?next=%s' % (settings.LOGIN_URL, request.path))
        if survey.need_logged_user and request.user.is_authenticated():
            if self.get_user_response(request.user, survey).exists():
                return redirect('survey-completed', id=survey.id)
        category_items = self.get_survey_categories(survey)
        categories = [c.name for c in category_items]
        form = ResponseForm(request.POST,
                            survey=survey,
                            user=request.user,
                            step=kwargs.get('step', 0))
        context = {
            'response_form': form,
            'survey': survey,
            'categories': categories
        }
        if form.is_valid():
            session_key = 'survey_%s' % (kwargs['id'],)
            if session_key not in request.session:
                request.session[session_key] = {}
            for key, value in form.cleaned_data.items():
                request.session[session_key][key] = value
                request.session.modified = True

            next_url = form.next_step_url()
            response = None
            if survey.display_by_question:
                if not form.has_next_step():
                    save_form = ResponseForm(request.session[session_key],
                                             survey=survey,
                                             user=request.user)
                    response = save_form.save()
            else:
                response = form.save()

            if next_url is not None:
                return redirect(next_url)
            else:
                del request.session[session_key]
                if response is None:
                    return redirect('/')
                else:
                    next = request.session.get('next', None)
                    if next is not None:
                        if 'next' in request.session:
                            del request.session['next']
                        return redirect(next)
                    else:
                        return redirect('survey-confirmation',
                                        uuid=response.interview_uuid)
        if survey.template is not None and len(survey.template) > 4:
            template_name = survey.template
        else:
            if survey.display_by_question:
                template_name = 'survey/survey.html'
            else:
                template_name = 'survey/one_page_survey.html'
        return render(request, template_name, context)


class ConfirmView(TemplateView):
    template_name = 'survey/confirm.html'

    def get_context_data(self, **kwargs):
        context = super(ConfirmView, self).get_context_data(**kwargs)
        context['uuid'] = kwargs['uuid']
        return context


class SurveyCompleted(TemplateView):
    template_name = 'survey/completed.html'

    def get_context_data(self, **kwargs):
        context = super(SurveyCompleted, self).get_context_data(**kwargs)
        survey = get_object_or_404(Survey, is_published=True, id=kwargs['id'])
        context['survey'] = survey
        return context



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
from .utils import validate_list
from django.conf import settings

class Survey(models.Model):
    name = models.CharField(max_length=400)
    description = models.TextField()
    is_published = models.BooleanField(default=False)
    need_logged_user = models.BooleanField(default=False)
    display_by_question = models.BooleanField(default=False)
    template = models.CharField(max_length=255, null=True, blank=True)

    class Meta:
        verbose_name = _('survey')
        verbose_name_plural = _('surveys')

    def __unicode__(self):
        return (self.name)

    @models.permalink
    def get_absolute_url(self):
        return ('survey-detail', [self.id])

    def questions(self):
        if self.pk:
            qs = Question.objects.filter(survey=self.pk)
            qs = qs.order_by('category__order', 'order')
        else:
            return Question.objects.none()


class Category(models.Model):
    name = models.CharField(max_length=400)
    survey = models.ForeignKey(Survey)
    order = models.IntegerField(blank=True, null=True)

    class Meta:
        verbose_name = _('category')
        verbose_name_plural = _('categories')

    def __unicode__(self):
        return (self.name)


class Question(models.Model):
    TEXT = 'text'
    SHORT_TEXT = 'short-text'
    RADIO = 'radio'
    SELECT = 'select'
    SELECT_IMAGE = 'select_image'
    SELECT_MULTIPLE = 'select-multiple'
    INTEGER = 'integer'

    QUESTION_TYPES = (
        (TEXT, _(u'text (multiple line)')),
        (SHORT_TEXT, _(u'short text (one line)')),
        (RADIO, _(u'radio')),
        (SELECT, _(u'select')),
        (SELECT_MULTIPLE, _(u'Select Multiple')),
        (SELECT_IMAGE, _(u'Select Image')),
        (INTEGER, _(u'integer')),
    )

    text = models.TextField()
    order = models.IntegerField()
    required = models.BooleanField(default=False)
    category = models.ForeignKey(Category, blank=True, null=True,)
    survey = models.ForeignKey(Survey)
    question_type = models.CharField(max_length=200,
                                     choices=QUESTION_TYPES,
                                     default=TEXT)
    # the choices field is only used if the question type
    choices = models.TextField(blank=True, null=True,
                               help_text=_(u"""if the question type is 'radio',
                                               'select', or 'select multiple'
                                               provide a comma-separated list
                                               of options for this question .
                                            """))

    class Meta:
        verbose_name = _('question')
        verbose_name_plural = _('questions')
        ordering = ('survey', 'order')

    def save(self, *args, **kwargs):
        separator = getattr(settings, 'SURVEY_SEPARATOR', ',')
        is_radio = self.question_type == Question.RADIO
        is_select = self.question_type == Question.SELECT
        is_select_multiple = self.question_type == Question.SELECT_MULTIPLE
        if (is_radio or is_select or is_select_multiple):
            validate_list(self.choices, separator=separator)
        super(Question, self).save(*args, **kwargs)

    def __unicode__(self):
        return (self.text)

    def get_choices(self):
        separator = getattr(settings, 'SURVEY_SEPARATOR', ',')
        return get_choices(self.choices)


class Response(models.Model):
    """
    A Response object is just a collection of questions and answers with a
    unique interview uuid
    """
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    survey = models.ForeignKey(Survey)
    user = models.ForeignKey(User, null=True, blank=True)
    interview_uuid = models.CharField(_(u"Interview unique identifier"),
                                      max_length=36)

    class Meta:
        verbose_name = _('response')
        verbose_name_plural = _('responses')

    def __unicode__(self):
        return ("response %s" % self.interview_uuid)


class AnswerBase(models.Model):
    question = models.ForeignKey(Question)
    response = models.ForeignKey(Response)
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

# these type-specific answer models use a text field to allow for flexible
# field sizes depending on the actual question this answer corresponds to. any
# "required" attribute will be enforced by the form.


class AnswerText(AnswerBase):
    body = models.TextField(blank=True, null=True)


class AnswerRadio(AnswerBase):
    body = models.TextField(blank=True, null=True)


class AnswerSelect(AnswerBase):
    body = models.TextField(blank=True, null=True)


class AnswerSelectMultiple(AnswerBase):
    body = models.TextField(blank=True, null=True)


class AnswerInteger(AnswerBase):
    body = models.IntegerField(blank=True, null=True)



"""Test for survey.utils module."""
from django.test import TestCase
from django.core.exceptions import ValidationError

from survey.utils import validate_list
from survey.utils import get_choices


class UtilsTestCase(TestCase):
    """Test for survey.utils functions."""

    def test_validate_list(self):
        """Test validate_list function."""
        self.assertEqual(validate_list("1,2"), None)
        self.assertEqual(validate_list("1,2,3"), None)
        self.assertEqual(validate_list("1;2;3", separator=';'), None)
        with self.assertRaises(ValidationError):
            validate_list("1")
        with self.assertRaises(ValidationError):
            validate_list("1", separator=';')

    def test_get_choices(self):
        """Test get_choices function."""
        self.assertEqual(
            get_choices("1,2"),
            (('1', '1'), ('2', '2'))
        )
        self.assertEqual(
            get_choices("a,b"),
            (('a', 'a'), ('b', 'b'))
        )
        self.assertEqual(
            get_choices("1;2", separator=";"),
            (('1', '1'), ('2', '2'))
        )



from django import template

register = template.Library()


class CounterNode(template.Node):

    def __init__(self):
        self.count = 0

    def render(self, context):
        self.count += 1
        return self.count


@register.tag
def counter(parser, token):
    return CounterNode()



import os


def local_path(path):
    os.path.join(os.path.dirname(__file__), path)

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'TEST_NAME': ':memory:'
    }
}

SITE_ID = 1

INSTALLED_APPS = [
    'django.contrib.contenttypes',
    'django.contrib.sites',
    'django.contrib.staticfiles',
    'django.contrib.auth',
    'django.contrib.admin',
    'survey',
    'tests',
]

MEDIA_URL = '/media/'

MEDIA_ROOT = local_path('media')

SECRET_KEY = "app-test"

TEMPLATE_DIRS = (
    local_path('templates'),
)

MIDDLEWARE_CLASSES = ()

ROOT_URLCONF = 'survey.urls'



#! -*- encoding: utf-8 -*-
from django import forms
from django.template.loader import render_to_string


class ImageSelectWidget(forms.widgets.Widget):

    class Media:
        js = (
            'http://ajax.googleapis.com/ajax/libs/jquery/1.6.4/jquery.min.js',
            'http://maps.googleapis.com/maps/api/js?sensor=false',
            'js/survey.js',
        )

    def render(self, name, value, *args, **kwargs):
        template_name = "survey/forms/image_select.html"
        choices = []
        for index, choice in enumerate(self.choices):
            if choice[0] != '':
                value, img_src = choice[0].split(":", 1)
                choices.append({
                    'img_src': img_src,
                    'value': value,
                    'full_value': choice[0],
                    'index': index
                })
        context = {'name': name, 'choices': choices}
        html = render_to_string(template_name, context)
        return html



"""Setup conf for tanuki app."""
from setuptools import setup, find_packages

setup(
    name="tanuki",
    version="0.1.2",
    author="Fabien Schwob",
    author_email="github@x-phuture.com",
    license="AGPL",
    url="https://github.com/jibaku/tanuki/",
    description="""Create dynamic forms in Django""".strip(),
    packages=find_packages(exclude=[]),
    include_package_data=True,
    install_requires=[
        'Django>=1.4',
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Framework :: Django",
    ]
)



"""Utils for taniku app."""

from django.core.exceptions import ValidationError


def validate_list(value, separator=','):
    """Take a text value and verifies that there is at least one comma."""
    values = value.split(separator)
    if len(values) < 2:
        msg = "The selected field requires an associated list of choices. "
        msg += "Choices must contain more than one item."
        raise ValidationError(msg)


def get_choices(value, separator=','):
    """
    Parse the choices and return a tuple.

    The tuple is formatted appropriately for the 'choices' argument
    of a form widget.
    """
    choices = value.split(separator)
    choices_list = []
    for c in choices:
        c = c.strip()
        choices_list.append((c, c))
    choices_tuple = tuple(choices_list)
    return choices_tuple
